import com.francisli.processing.http.*;

/*
The q search term can also contain any combination of the supported repository search qualifiers:

in: Qualifies which fields are searched. With this qualifier you can restrict the search to just the repository name, description, readme, or any combination of these.
size:  Finds repositories that match a certain size (in kilobytes).
forks:  Filters repositories based on the number of forks, and/or whether forked repositories should be included in the results at all.
created: or pushed: Filters repositories based on date of creation, or when they were last updated.
user: or repo: Limits searches to a specific user or repository.
language: Searches repositories based on the language they’re written in.
stars: Searches repositories based on the number of stars

q  string  The search keywords, as well as any qualifiers.
sort:  string  The sort field. One of stars, forks, or updated. Default: results are sorted by best match.
order:  string  The sort order if sort parameter is provided. One of asc or desc. Default: desc

*/
private static String baseURL = "https://api.github.com/search/repositories?";
private static String OAuthToken = "access_token=cedef3776cb0f2b68a17c1f48343fb8db9dce297";

//Repository Statistics
private static String repoBaseURL = "https://api.github.com/repos/"; //owner/repo/stats

// stats options
/* punch card ARRAY
Each array contains the day number, hour number, and number of commits:
0-6: Sunday - Saturday
0-23: Hour of day
Number of commits
For example, [2, 14, 25] indicates that there were 25 total commits, during the 2:00pm hour on Tuesdays. All times are based on the time zone of individual commits.
*/
private static String punch_card ="punch_card"; 
/* participation  OBJECT
Returns the total commit counts for the owner and total commit counts in all. all is everyone combined, including the owner in the last 52 weeks. If you’d like to get the commit counts for non-owners, you can subtract owner from all.

The array order is oldest week (index 0) to most recent week.
*/
private static String participation = "participation";

/* code_frequency ARRAY
Returns a weekly aggregate of the number of additions and deletions pushed to a repository.
*/
private static String code_frequency = "code_frequency";

/* commit_activity =ARRAY
Returns the last year of commit activity grouped by week. The days array is a group of commits per day, starting on Sunday.
*/
private static String commit_activity = "commit_activity";

private static String[] keywords = {"data+visualization", "interactive+visualization", "infographic"};

JSONArray data;
int dataCounter =0;

void setup(){
  data = new JSONArray();
  for (String keyword: keywords){
  try{
  search(keyword);
  }
  catch (Exception e)  {
    e.printStackTrace();  
  }
  }
saveJSONArray(data, "data/new.json");
}
private void search(String keyword) throws Exception{
String order = "&order=desc";
String sort = "&sort=stars";
String query ="&q="+keyword;

String searchURL = baseURL+OAuthToken+query+sort+order;
JSONObject  root = loadJSONObject(searchURL);   
//println(root);
JSONArray items = root.getJSONArray("items");
//println("Result count: "+ items.size());
for (int i=0;i<items.size();i++){
  try {
  JSONObject  item = items.getJSONObject(i);
    String name =item.getString("name");
    //String language = item.getString("language");
    //int size = item.getInt("size");
    JSONObject  owner = item.getJSONObject("owner");
    String login = owner.getString("login");
    //String avatar_url = owner.getString("avatar_url");
    JSONObject repository = new JSONObject();
    
    repository.setString("name",name);
    repository.setString("language",item.getString("language"));
    repository.setString("owner",login);
    repository.setString("avatar_url",owner.getString("avatar_url"));
    repository.setInt("size",item.getInt("size"));
    repository.setInt("watchers_count",item.getInt("watchers_count"));
    repository.setJSONArray("code_frequency",getRepoStats(login,name,code_frequency));
    repository.setJSONArray("commit_activity",getRepoStats(login,name,commit_activity));
    repository.setJSONArray("punch_card",getRepoStats(login,name,punch_card));
    data.setJSONObject(dataCounter,repository);
    dataCounter++;
    //println("===========");
    //println("KeyWord: "+keyword);
    //println("Name: "+name);
    //println("Language: "+language);
    //println("Owner: "+login);
    
  }   catch (Exception e)  {
    e.printStackTrace();  
  }   
  }
}

private JSONArray getRepoStats(String owner, String repo, String searchType){
  String url = repoBaseURL+owner+"/"+repo+"/stats/"+searchType;
  int counter =0;
  JSONArray root = new JSONArray();
  do{
  root = loadJSONArray(url);   
  counter++;
  } while (root.size() == 0 || counter<4);
  println(root); 
  return root;
}

private JSONObject getRepoParticipation(String owner, String repo, String searchType){
  String url = repoBaseURL+owner+"/"+repo+"/stats/"+searchType+"&"+OAuthToken;
  int counter =0;
  JSONObject root = new JSONObject();
  do{
  root = loadJSONObject(url);   
  counter++;
  } while (root.size() == 0 || counter<4);
  println(root); 
  return root;
}
